/* 
 * redirect.cpp
 *
 *  This file implements the redirection of stadard output and standard error
 *  using the stream classes.
 *
 * System: Hush for MS-Windows
 * Author: J.P. Alten
 * Last modified on Feb. 12 1996
 *
 */

#pragma hdrstop

#include "redirect.h"

infostreambuf::infostreambuf(messageproc proc) :
    streambuf()
{
    buffer = new char[bufsize=30];  // create a buffer for the data
    textlen=0;                      // no data yet
    showmessage = proc;             // set the procedure which will handle the output
}

// Destructor
infostreambuf::~infostreambuf()
{
    flush();            // write all pending data to output
    delete[] buffer;    // free the resources
}


void infostreambuf::flush( )
{
    if (textlen==0)      // anything to write?
        return;

    buffer[textlen]=0;   // set the end of string marker
    showmessage(buffer); // write all pending data to output
    textlen=0;           // reset the buffer length: no more data
}

int infostreambuf::overflow( int c )
{
    switch (c)
    {
        case '\r' : break;          // do nothing
        case '\n' : flush(); break; // show message
        case '\t' : do              // fill with spaces
                        overflow(' ');
                    while (textlen%4 >0);
                    break;
        default:
        {                           // add one character, resize buffer if needed
            if (textlen>=bufsize-2)
            {
                bufsize+=50;
                char* newbuf = new char[bufsize];
                memcpy(newbuf,buffer,textlen);
                delete[] buffer;
                buffer = newbuf;
            }
            buffer[textlen++]=c;
        }
    }
    return 1;
}



// This requires a ShowMessage procedure to be declared somewhere else in the program
// extern "C" void ShowMessage(const char* buffer);
