// tclsession.c  -- (no session.c because of namemangling conflicts)
// Thanks to Richard A. Rausch

#include <dv/tcl/session.h>
#include <dv/tcl/kit.h>

#include <hush/action.h>
#include <hush/widget.h>
#include <hush/event.h>

#include <hush/assert.h>

#include <hush/config.h>


#include <dv/flags.h>
#include <dv/port.h>

#include <dv/tcl/callback.h>
#include <dv/tcl/file.h>
#include <dv/tcl/action.h>


#include <stdlib.h>


#ifdef __MSWIN__
#include <io.h>
#else
#include <unistd.h>
#endif // __MSWIN__


#include <string.h>

/// Change this for a different tk file

extern "C" {
#include <tcl.h>
#include <tk.h>
#include <tkInt.h>
#ifdef MULTINTERP
#if (TK_MAJOR_VERSION >= 4 && TK_MINOR_VERSION < 1)
#include <interpCmd.h>
#endif // VERSION
#endif // MULTINTERP
}

struct TkWindow;


typedef void timerproc( ClientData );
void init_tk(kit*); /// defined below

#define INTERP (Tcl_Interp*) _interp

extern int tcl_appinit(kit*);

static int quit_flag = 0;

#ifndef __MSWIN__           // jpa
static void* global_interp = 0;
static void* _mainwindow = 0;
#else
void* global_interp = 0;
void* _mainwindow = 0;
#endif

static kit* _thekit = 0;
static int _tclactno = 0;

static int _trace = 0;
static widget* _theroot = 0;
static widget* _thecanvas = 0;
static char _buf[BUFSIZ];

static char hush_library[BUFSIZ];
static char tcl_library[BUFSIZ];
static char tk_library[BUFSIZ];

void* theinterp() { return ::global_interp; }
void theinterp(void* x) { ::global_interp = x; }

void the_canvas(widget* x) { ::_thecanvas = x; }
widget* the_canvas() { return ::_thecanvas; }

kit* thekit() { return ::_thekit; }
void thekit(kit *k) { ::_thekit=k;}

static widget* theroot() { require(::_theroot); return ::_theroot; }

void* themainwindow() { return ::_mainwindow; }
void themainwindow(void* x) { ::_mainwindow = x; }

void* thewindow(kit* tk, const char* path) {
Tk_Window mwin = (Tk_Window) themainwindow();
Tk_Window win = Tk_NameToWindow((Tcl_Interp*)tk->interp(), (char*) path, mwin);
return (void*)  win;
}

//// tcl_callbacks

static tcl_callback* active_callback;
static void* last_callback() { return (void*)active_callback; }

static int call_back(ClientData c, Tcl_Interp*, int argc, char** argv) {
tcl_callback* x = ((tcl_callback*) c);
active_callback = x; 
return (*((hushcommand*)x->proc))(x->data, x->tk,argc,argv);
}



//// hush::tcl_kit

int tcl_kit::n = 0;

tcl_kit::tcl_kit(void* x) : kit("tcl") {
        _interp = (Tcl_Interp*) x;
	//cerr << "COPYING SLAVE" << endl;
	}
tcl_kit::tcl_kit() : kit(this) {
	_trace = 0;
	if (n==0) {
		//cerr << "CREATING MAIN" << endl;
		//::_interp = _interp = (void*) Tcl_CreateInterp();
		::global_interp = _interp = ::global_interp?
		   (::global_interp):(void*) Tcl_CreateInterp();  // jpa

		n++;
		}
	else {
		//cerr << "COPYING MAIN" << endl;
		if (_trace > 6) cerr << "interp" << ++n << endl;
		_interp = ::global_interp;
		}

	buf = new char[4*1024];
	_root = 0;
	promise ( _interp );
	promise ( ::global_interp );
}

tcl_kit::~tcl_kit() { delete buf; }


int tcl_kit::eval(const char* cmd) {
	if (_trace) cerr << (char*) cmd << endl;
	int err = Tcl_Eval(INTERP,(char*)cmd);
	// require ( err == TCL_OK );
	return err;
	}

char* tcl_kit::evaluate(const char* cmd) {
	eval(cmd);
	return result();
	}

char* tcl_kit::result() const { return (INTERP)->result; }

void tcl_kit::result( const char* s) {
	Tcl_SetResult(INTERP, (char*)s, TCL_VOLATILE);
	}

int tcl_kit::source(const char* f) {
	return Tcl_EvalFile(INTERP, (char*) f);
	}

int tcl_kit::error() { return (INTERP)->errorLine; }

void tcl_kit::error(const char* msg) {
	Tcl_AddErrorInfo(INTERP, (char*) msg);
	}

void tcl_kit::after(int t, const char* cmd) {
	sprintf(buf,"after %d %s",t,cmd);
	eval(buf);
	}

void tcl_kit::after(int t, void* f, void* d) {
	Tk_CreateTimerHandler(t, (timerproc*) f, (ClientData) d);
	}

void tcl_kit::after(int t, handler* h) {
	struct tcl_callback* x = new tcl_callback();
	x->tk = this; x->proc = (void*) handler_callback; x->data = h;
	after(t, (void*) timer_callback, (void*) x);
	}
	

char* tcl_kit::send(const char* it, const char* cmd) {
	sprintf(buf,"send %s %s",it,cmd);
        return evaluate(buf);
	}

char* tcl_kit::selection(const char* options) const {
	sprintf(buf,"selection get %s", options);
	return ((tcl_kit*)this)->evaluate( buf );
	}

void tcl_kit::update(const char* options) { 
	sprintf(buf,"update %s", options);
	eval( buf );
	}


int tcl_kit::safe() const {
#ifdef MULTINTERP
	int res = Tcl_IsSafe(INTERP);
#else
	int res = 0;
#endif
	return res;
	}

kit* tcl_kit::slave(char* options) {
	kit* res = 0;
	char cmd[BUFSIZ];
	sprintf(cmd,"interp create %s ",options);
	int err = this->eval( cmd );
	if (err) { 
		cerr << "ERROR: CREATING INTERPRETER " << endl;
	} else {
#ifdef MULTINTERP
	   void* x = Tcl_GetSlave(INTERP, this->result() );
	   res =  new tcl_kit(x);
#endif
	}
	promise( res );
	return res;
}

class action* tcl_kit::action(handler* h) {
	return action(gensym("anonymous"),h);
	}
class action* tcl_kit::action(void* proc, void* data) {
	return action(gensym("anonymous"),proc,data);
	}

class action* tcl_kit::action(const char* name) {
	return new tcl_action(this,name);
	}

class action* tcl_kit::action(const char* name, handler* h) {
	return action(name, (void*) handler_callback, h );
	}

class action* tcl_kit::action(const char* name, void* proc, void* data) {
	struct tcl_callback* x = new struct tcl_callback();
	x->tk = this; x->proc = proc; x->data = data;
	command(name,(void*)call_back,(void*)x);
	return new tcl_action(this,name);
	}

void tcl_kit::command(const char* name, void* proc, void* data = 0) {
	Tcl_CreateCommand(INTERP,(char*) name, 
	(Tcl_CmdProc*) proc, (ClientData) data,
	(Tcl_CmdDeleteProc*) 0);
	}

void tcl_kit::quit() { eval("quit"); ::quit_flag = 1; }

void tcl_kit::trace(int i) { ::_trace = _trace = i; }
void tcl_kit::notrace() { ::_trace = _trace = 0; }


event* tcl_kit::get() const {
	if (_trace > 3 ) cerr << "asking for event" << endl;
	event* x = (event*) _last;
#ifdef GNU
	return event( (event*) x );
#else
	return new event( (event*) x );
#endif
	}


widget* tcl_kit::root() const  {
	if (!_root) ((tcl_kit*)this)->_root = new widget(".");
	require( _root );
	return _root;
	}

widget* tcl_kit::pack(widget* w, const char* options) {
        if (tracelevel()>5) cerr << "packing to root (W)" << endl;
	return root()->pack(w,options);
	}

widget* tcl_kit::pack(const char* w, const char* options) {
        if (tracelevel()>5) cerr << "packing to root (*)" << endl;
	return root()->pack(w,options);
	}

int tcl_kit::callback(void*, void*, int, char**) { return 0; } //AE

void* tcl_kit::interp() const { promise(_interp); return _interp; }


#ifndef HCLONLY
void* tcl_kit::tkwin() const {
	return (void*) _mainwindow;
	}
#endif


void tcl_kit::put(event* p, int fl) {
#ifndef HCLONLY
	require( p );
	_last = p;
	_type = fl;
#endif
}


/////////////// session Session SESSION tcl_session


tcl_session::tcl_session(int argc_, char** argv_, const char* name, int fl) : session(this) {  
	_argc = argc_;
	_argv = argv_;
	_name = (char*) name;
	_init = 0;
	_flag = fl;
	_mode = 0;

	if (!name) _flag = 1;

	const char* s = getenv("HUSH_LIBRARY");
	if (s == NULL)
		sprintf(::hush_library, "%s", HUSH_LIBRARY);
	else
		sprintf(::hush_library, "%s", s);
	}

void tcl_session::init(const char* f, void*) {  
	_init = new char[ strlen(f) + 1 ];
        strcpy(_init, f);
	}

void tcl_session::install(void* p) {
	_session = (session*) p;
	::_thekit = tk = _session->thekit(); //TODO tk = ?
	promise( ::_thekit && tk && _session );
	}



Tcl_Interp* tk_mainInterp;

#ifndef HCLONLY
static Tk_Window MainWindow;

static char *hushrc_filename = "~/.hushrc";

static Tcl_DString buffer;
static int tty;

static int code;


//  Command-line options:

static int synchronize = 0;

static char *filename = NULL;
static char *appname = NULL;
static char *display = NULL;
static char *geometry = NULL;

static char* hush_name = "hush";

static int _nowin = 0;
#endif



#ifndef HCLONLY
Tk_ArgvInfo argTable[] = {
    {"-file", TK_ARGV_STRING, (char *) NULL, (char *) &filename,
	"File from which to read commands"},
    {"-geometry", TK_ARGV_STRING, (char *) NULL, (char *) &geometry,
	"Initial geometry for window"},
    {"-display", TK_ARGV_STRING, (char *) NULL, (char *) &display,
	"Display to use"},
    {"-name", TK_ARGV_STRING, (char *) NULL, (char *) &appname,
	"Name to use for application"},
    {"-sync", TK_ARGV_CONSTANT, (char *) 1, (char *) &synchronize,
	"Use synchronous mode for display server"},

    {"-x", TK_ARGV_CONSTANT, (char *) 1, (char *) &_nowin,
	"Use synchronous mode for display server"},


    {(char *) NULL, TK_ARGV_END, (char *) NULL, (char *) NULL,
	(char *) NULL}
};

static void    Prompt _ANSI_ARGS_((Tcl_Interp *interp, int partial));
static void    StdinProc _ANSI_ARGS_((ClientData clientData, int mask));

#endif /* HCLONLY */



#ifndef HCLONLY
static void*  the_last_xevent = 0;
static void put_last_x_event(void* last) {
	the_last_xevent = last;
	}

extern void* get_last_x_event() {
	void* tmp = the_last_xevent;
	the_last_xevent = 0;
	return tmp;
	}

int traceX(ClientData k, XEvent* last) {
put_last_x_event((void*) last);
//((kit*)k)->put((event*) last, event::x_event );
return 0;
}
#endif


// In order to interprete files with -f (and still go-back )

static int callprog = 1;

int
gobackCmd ( // tkwin, interp, argc, argv)
    ClientData ,  /* Application window. */
    Tcl_Interp *,	     /* Current interpreter. */
    int ,		     /* Number of arguments. */
    char **)	     /* Argument strings. */
{


    callprog = 1;
    return TCL_OK;
}

int
quitCmd ( // tkwin, interp, argc, argv)
    ClientData ,  /* Application window. */
    Tcl_Interp *,	     /* Current interpreter. */
    int ,		     /* Number of arguments. */
    char **)	     /* Argument strings. */
{


    cerr << "Quitting" << endl;
    exit(0);
    ::quit_flag = 1;
    return TCL_OK;
}

// run map loop body

#ifndef HCLONLY

void tcl_session::map(int fl) {
	
    if (::_trace > 6) cerr << "tcl_kit::map " << fl <<endl;
    Tcl_Interp* interp = (Tcl_Interp*) ::global_interp; // tk->interp();

    require(interp);

    if (!fl) {
    MainWindow = ::_mainwindow?
	 (Tk_Window)(::_mainwindow) :
	 Tk_CreateMainWindow(interp, display, appname,"Tk"); // jpa
    ::_mainwindow = (void*) MainWindow;

    if (MainWindow == NULL) {
	cerr<< interp->result << endl;      // jpa
	exit(1);
    }
    Tk_SetClass(MainWindow, "Tk");
#ifndef __MSWIN__                     // jpa
    if (synchronize) {
	XSynchronize(Tk_Display(MainWindow), True);
    }
#endif
    Tk_GeometryRequest(MainWindow, 50, 50);
    }
//return 0;
}

/// the mainloop, MainLoop MAINLOOP

#if (TK_MAJOR_VERSION >= 4 && TK_MINOR_VERSION < 1)
extern int tk_NumMainWindows;
#endif

//tcl_session::loop(int) { Tk_MainLoop(); return 0; }

int tcl_session::mode() const { return _mode; }
int tcl_session::mode(int i) { int tmp = _mode; _mode = i; return tmp; }
int tcl_session::mode(const char* s) {
	int tmp = _mode;
	if (!strcmp(s,"real")) _mode = TK_DONT_WAIT;
	else if (!strcmp(s,"dont_wait")) _mode = TK_DONT_WAIT;
	else _mode = 0;
	return _mode;
	}


int tcl_session::body(int fl) {
 // cerr << "BODY:" << endl;
return fl;
}

tcl_session::loop(int fl) {
// cerr << "START LOOPING:" << endl;
       int finish = 0;
       while ( !quit_flag && ( !finish && // 0 or TK_DONT_WAIT
		( fl || (
#if (TK_MAJOR_VERSION >= 4 && TK_MINOR_VERSION < 1 )
		tk_NumMainWindows
#else
		Tk_GetNumMainWindows()
#endif
		> 0)) )) {
	   while ( !quit_flag && Tk_DoOneEvent( _mode ) )
				if (quit_flag) return quit_flag; 

	   finish = _session->body(fl);
       }
return finish;
}

//We discovered (the hard way) that it is essential to process 
//all pending events before relinquishing the thread otherwise
//events can built up faster than they get processed.

//In our system we allow only the thread scheduler to relinquish 
//the Unix process (other than by preemption), so the TK_DONT_WAIT was 
//added to prevent wish from blocking.  If there are any other
//blocking i/o functions in wish then we will need to modify those
//too, but we haven't found any yet.

//Hope this helps.   John Ellson AT&T Bell Labs


tcl_session::run(int _proc(class kit*,int,char**)) {
char *args, *p, *msg;
char buf[20];
int result = 1001;
//Tk_3DBorder border;

int argc = _argc;
char** argv = _argv;


extern char *exp_argv0; // for Expect


    require( tk );
    ::_thekit = tk;

    Tcl_Interp* interp = (Tcl_Interp*) ::global_interp;
    require( interp );

    if (!::_nowin) init_tk(tk);

    Tcl_SetVar(interp, "dummy", "DUMMY", TCL_GLOBAL_ONLY);

    /*
    if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv, argTable, 0)
	    != TCL_OK) {
	cerr<< interp->result << endl;      // jpa
	exit(1);
    }
    */

 // Recognize _single_ filename argument (5/31/95, jrvosse@cs.vu.nl):
    if (argc == 2 && argv[1][0] != '-') {
        filename = argv[1];
    }


    while ( argc > 0 ) {
	if (argv[0][0]=='-') {
	switch (argv[0][1]) {
		case 'f': 
			filename = argv[1];
			argv += 2; argc -= 2;
			break;
		case '-':
		case 'p':
		case 'x':
			argc--; argv++; break;
			//argv += 2; argc -= 2;
		default:
			cerr << "No such option " << argv[0] << endl;
			argc--; argv++;
			break;
		}
	} else {
	argc--; argv++;
	}
      }
resume:

    if (appname == NULL) appname = _name;
    if (appname == 0) appname = hush_name;

    if (0 && appname == NULL) {
	if (filename != NULL) {
	    p = filename;
	} else {
	    p = argv[0];
	}
	appname = strrchr(p, '/');
	if (appname != NULL) {
	    appname++;
	} else {
	    appname = p;
	}
    }

   if (display != NULL) {
	Tcl_SetVar2(interp, "env", "DISPLAY", display, TCL_GLOBAL_ONLY);
	}


    /*
     * Initialize the Tk application and arrange to map the main window
     * after the startup script has been executed, if any.  This way
     * the script can withdraw the window so it isn't ever mapped
     * at all.
     */
    
    tty = isatty(0);

    if (!::_nowin) _session->map(_flag);
    //if (!::_nowin) ::_theroot = new widget(".");

    //promise( theroot() );
    if (!::_nowin) init_widgets();	
	   // needs root window, should be after map()

    /*
     * Make command-line arguments available in the Tcl variables "argc"
     * and "argv".  Also set the "geometry" variable from the geometry
     * specified on the command line.
     */

    args = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
    ckfree(args);
    sprintf(buf, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "argv0", (filename != NULL) ? filename : _argv[0],
	TCL_GLOBAL_ONLY);
    if ((geometry != NULL)) {
	Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
    }

    Tcl_SetVar(interp, "tcl_interactive",
		    ((filename == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);


    /*
     * Add a few application-specific commands to the application's
     * interpreter.
     */

    /*
    if (Tcl_AppInit(interp) != TCL_OK) {
	cerr << "Tcl_AppInit failed: " << interp->result << endl;   //
    }
    */

    tcl_appinit(tk);

    if (!_init) {
	//sprintf(::_buf,"source $hush_library/init.tcl");
	//tk->eval( ::_buf );
    } else {
		sprintf(::_buf,"%s",_init);
		tk->eval( ::_buf );
		}

    if ((geometry != NULL)) {
       code = Tcl_VarEval(interp, "wm geometry . ", geometry, (char *) NULL);
       if (code != TCL_OK) {
	   cerr << interp->result << endl;          // jpa
       }
    }



    tk->command("go-back", (void*) gobackCmd,0);
    tk->command("quit", (void*) quitCmd,0);


    //tk->eval("button .quit -text quit -command { destroy . }");
    //if (result != TCL_OK) { goto error; }

    Tk_CreateGenericHandler(traceX,(ClientData)tk);

    //tk->install(tk,argc,argv);
    //if (callprog) tk->program(tk,argc,argv); // ** this is where it happens

    _session->prelude(); //

    (void) Tcl_Eval(interp, "update");
    

    if (filename != NULL) {
	callprog = 0; // HUSH --> may be overruled by go-back
	code = Tcl_VarEval(interp, "source ", filename, (char *) NULL);
	if (code != TCL_OK) {
	    goto error;
	}
	tty = 0;
    } else {
#ifndef __MSWIN__
	/*
	 * Commands will come from standard input, so set up an event
	 * handler for standard input.  If the input device is aEvaluate the
	 * .rc file, if one has been specified, set up an event handler
	 * for standard input, and print a prompt if the input
	 * device is a terminal.
	 */

	if (hushrc_filename != NULL) {
	    Tcl_DString buffer;
	    char *fullName;
    
	    fullName = Tcl_TildeSubst(interp, hushrc_filename, &buffer);
	    if (fullName == NULL) {
		cerr << interp->result << endl;         // jpa
	    } else {
		if (access(fullName, 4) == 0) { // R_OK
		    code = Tcl_EvalFile(interp, fullName);
		    if (code != TCL_OK) {
			cerr << interp->result << endl;     // jpa
		    }
		}
	    }
	    Tcl_DStringFree(&buffer);
	}
	Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) 0);
	if (tty) {
	    Prompt(interp, 0);
	}
#endif /* __MSWIN__ */
    }
    cout.flush(); // jpa
    Tcl_DStringInit(&buffer);
    (void) Tcl_Eval(interp, "update");

    /*
     * Loop infinitely, waiting for commands to execute.  When there
     * are no windows left, Tk_MainLoop returns and we clean up and
     * exit.
     */


    //if (callprog) tk->program(tk,_argc,_argv); // ** this is where it happens

    if (callprog) {
	if (_proc)
		result = _proc(tk,_argc,_argv);
	else
	        result = _session->main();
	}
    
    //while ( Tk_DoOneEvent(0) ) ;
    //Tk_MainLoop();

    _session->loop(_flag);

    //Tcl_DeleteInterp(interp);
    Tcl_DStringFree(&buffer);
    //tk->eval("exit");
    //exit(0);
    return( result );
   

error:
    msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
    if (msg == NULL) {
	msg = interp->result;
    }
    cerr << msg << endl;
    //Tcl_Eval(interp, "destroy .");
    //exit(1);
    return 0;			/* Needed only to prevent compiler warnings. */
}

#endif /* HCLONLY */




#ifndef HCLONLY
#ifndef __MSWIN__            // jpa

/*
 *----------------------------------------------------------------------
 *
 * StdinProc --
 *
 *	This procedure is invoked by the event dispatcher whenever
 *	standard input becomes readable.  It grabs the next line of
 *	input characters, adds them to a command being assembled, and
 *	executes the command if it's complete.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Could be almost arbitrary, depending on the command that's
 *	typed.
 *
 *----------------------------------------------------------------------
 */

    /* ARGSUSED */
static void
StdinProc( //clientData, mask)
    ClientData ,		/* Not used. */
    int )				/* Not used. */
{
#define BUFFER_SIZE 4000
    char input[BUFFER_SIZE+1];
    static int gotPartial = 0;
    char *cmd;
    int code, count;
    Tcl_Interp* interp = (Tcl_Interp*) ::global_interp;


    count = read(fileno(stdin), input, BUFFER_SIZE);
    if (count <= 0) {
	if (!gotPartial) {
	    if (tty) {
		Tcl_Eval(interp, "exit");
		exit(0);
	    } else {
		Tk_DeleteFileHandler(0);
	    }
	    return;
	} else {
	    count = 0;
	}
    }
    cmd = Tcl_DStringAppend(&buffer, input, count);
    if (count != 0) {
	if ((input[count-1] != '\n') && (input[count-1] != ';')) {
	    gotPartial = 1;
	    goto prompt;
	}
	if (!Tcl_CommandComplete(cmd)) {
	    gotPartial = 1;
	    goto prompt;
	}
    }
    gotPartial = 0;

    /*
     * Disable the stdin file handler while evaluating the command;
     * otherwise if the command re-enters the event loop we might
     * process commands from stdin before the current command is
     * finished.  Among other things, this will trash the text of the
     * command being evaluated.
     */

    Tk_CreateFileHandler(0, 0, StdinProc, (ClientData) 0);
    code = Tcl_RecordAndEval(interp, cmd, 0);
    Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) 0);
    Tcl_DStringFree(&buffer);
    if (*interp->result != 0) {
	if ((code != TCL_OK) || (tty)) {
	    cout << interp->result << endl;
	}
    }

    /*
     * Output a prompt.
     */

    prompt:
    if (tty) {
	Prompt(interp, gotPartial);
    }
}
#endif /* __MSWIN__ */

/*
 *----------------------------------------------------------------------
 *
 * Prompt --
 *
 *	Issue a prompt on standard output, or invoke a script
 *	to issue the prompt.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	A prompt gets output, and a Tcl script may be evaluated
 *	in interp.
 *
 *----------------------------------------------------------------------
 */

static void
Prompt(// interp, partial)
    Tcl_Interp *interp,			/* Interpreter to use for prompting. */
    int partial)			/* Non-zero means there already
					 * exists a partial command, so use
					 * the secondary prompt. */
{
    char *promptCmd;
    int code;

    promptCmd = Tcl_GetVar(interp,
	partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
    if (promptCmd == NULL) {
	defaultPrompt:
	if (!partial) {
	     cout  << "% " ;                 // jpa
	}
    } else {
	code = Tcl_Eval(interp, promptCmd);
	if (code != TCL_OK) {
	    Tcl_AddErrorInfo(interp,
		    "\n    (script that generates prompt)");
	    cerr << interp->result << endl;         // jpa
	    goto defaultPrompt;
	}
    }
    cout.flush();                                // jpa
}

#endif /* HCLONLY */









#ifndef HCLONLY

/*
 * SET_CLASS -- add setclass command
 */

       /* ARGSUSED */
int
SetClassCmd(
    ClientData , 
    Tcl_Interp *interp,
    int argc,
    char **argv)
{
    if (argc != 3) {
      Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
                       " window classname\"", (char *) NULL);
      return TCL_ERROR;
    }
    Tk_SetClass(Tk_NameToWindow(interp,argv[1],Tk_MainWindow(interp)), argv[2]);

    return TCL_OK;
}

void init_tk( kit* tk ) {
// tk->action("bell", (void*) Tk_BellCmd, (void*) MainWindow );
// tk->action("dot", (void*) Tk_DotCmd, (void*) MainWindow );
// tk->action("moveto",(void*) Tk_MovetoCmd, (void*) MainWindow );
// tk->action("lineto", (void*) Tk_LinetoCmd, (void*) MainWindow );
// tk->action("setclass", (void*) SetClassCmd, (void*) MainWindow);
}

#else

void init_tk(kit*) { }

#endif /* HCLONLY */


int tcl_appinit(kit* tk) {
char buf[BUFSIZ];

require( tk );

// tk->trace(7);

char* s = getenv("HUSH_LIBRARY");

if (s == NULL)
	sprintf(::hush_library, "%s", HUSH_LIBRARY);
else
	sprintf(::hush_library, "%s", s);

char* tcl_d = getenv("TCL_LIBRARY");
if (tcl_d == NULL)
	sprintf(::tcl_library, "%s",TCL_LIBRARY);
else
	sprintf(::tcl_library, "%s", tcl_d);
	
    
char* tk_d = getenv("TK_LIBRARY");

if (tk_d == NULL)
	sprintf(::tk_library, "%s",TK_LIBRARY);
else
	sprintf(::tk_library, "%s", tk_d);


sprintf(buf,"set hush_library %s",::hush_library);
tk->eval(buf );

sprintf(buf,"set tcl_library %s",::tcl_library);
tk->eval(buf );

sprintf(buf,"set tk_library %s",::tk_library);
tk->eval(buf );


if (1) {
sprintf(buf,"source $tcl_library/%s","init.tcl");
tk->eval(buf );

sprintf(buf,"source $tk_library/%s","tk.tcl");
tk->eval(buf );
}

if (1) {
sprintf(buf,"source $hush_library/%s","init.tcl");
tk->eval(buf );
}

return 0;

}

int Tcl_AppInit(Tcl_Interp* interp) {
require(interp);
#ifdef __MSWIN__
#ifdef lethushdotcl
     int Console_Init(Tcl_Interp*);
     Tcl_SetVar(interp, "tcl_interactive", "1" ,
         TCL_GLOBAL_ONLY);
     Tcl_SetVar2(interp, "env", "DISPLAY", "localhost:0" ,
         TCL_GLOBAL_ONLY);
     Console_Init(interp);
#endif
#endif                          // jpa

return TCL_OK;
}

/// I do find the Tcl/Tk code somewhat messy
/// On the other hand, at times I look with disregard at mine as well



void register_as(const char* path, const widget* w) {

    Tcl_Interp *interp = (Tcl_Interp*) theinterp();
    TkWindow* tkwin = (TkWindow*) w->tkwin(); 
    TkWindow* win = (TkWindow*) w->tkwin();
    Tcl_HashEntry *hPtr;
    int c_new;
 
    hPtr = Tcl_CreateHashEntry(& tkwin->mainPtr->nameTable,(char*) path, &c_new);
    if (!c_new)
     	cerr << "WINDOW " << path << " " << "exists" << endl;
    Tcl_SetHashValue(hPtr, win);
    win->pathName = Tcl_GetHashKey( & tkwin->mainPtr->nameTable, hPtr);


}
    
// jrvosse
void tcl_kit::command(int fd, void* proc, void* data, int mask) {
    Tk_CreateFileHandler(fd, mask, (Tk_FileProc*)proc, (ClientData)data);
}

class action* tcl_kit::action(int fd, handler* h, int mask) {
    file_callback *s = new file_callback(h,fd);
    command(fd, (void*) file_handler_callback,(void*) s, mask);
    return new tcl_action(this,"nop");
}

void tcl_kit::unbind(int fd) {
	Tk_DeleteFileHandler(fd);
	}

void tcl_kit::unbind(char* name, char* newname) {
	cerr << "ERROR: no unbind for " << name << endl;
	}
