#include <dv/tcl/x.h>

#include <dv/tcl/callback.h>
#include <dv/tcl/event.h>

#include <hush/assert.h>

#ifndef HCLONLY
extern "C" {
#include <X11/Xlib.h>
}
#endif

#define LAST ((XEvent*) last)

static char* xeventnames[] = {
"UserEvent",
"TclEvent",
"KeyPress",
"KeyRelease",
"ButtonPress",
"ButtonRelease",
"MotionNotify",
"EnterNotify", 
"LeaveNotify",
"FocusIn",   
"FocusOut", 
"KeymapNotify", 
"Expose",      
"GraphicsExpose", 
"NoExpose",      
"VisibilityNotify",
"CreateNotify",   
"DestroyNotify", 
"UnmapNotify",  
"MapNotify",   
"MapRequest", 
"ReparentNotify",
"ConfigureNotify",
"ConfigureRequest",  
"GravityNotify",    
"ResizeRequest",    
"CirculateNotify", //       26"
"CirculateRequest", //        27"
"PropertyNotify", //          28"
"SelectionClear", //          29"
"SelectionRequest", //        30"
"SelectionNotify", //          31"
"ColormapNotify", //          32"
"ClientMessage", //           33"
"MappingNotify", //           34"
"LASTEvent" //               35      /* must be bigger than any event # */"
};





static char* xeventname(int i) { return xeventnames[i]; }

class widget;
class callback;
class client;


#ifndef HCLONLY
xevent::xevent(void* p) : event(this) { last = p; promise(last); }

xevent::xevent(class kit* _tk, int _argc, char* _argv[]) {
        tk = _tk;
        __argc = _argc;
        __argv = _argv;
        last = get_last_x_event(); 
        }

xevent::~xevent() { }

int xevent::argc() const { return __argc; }
char* xevent::arg(int i ) const { return  argv()[i]; }
char** xevent::argv(int i) const { return  __argv+i; }


char* xevent::info() const { return "x_event"; }
int xevent::eventclass() const { return (int) event::tcl; }
int xevent::type() const { return last?LAST->type:0; }

char* xevent::name() const { return xeventname(type()); } 
void xevent::trace() const {
	cerr << name() << "(" << x() << "," << y() << ")" << endl;
	}
// AE union XEvent* xevent::XEvent() { return last; }
void* xevent::rawevent() { return last; }


void* xevent::callback() { return cb; }

//class widget* xevent::widget() { return cb->w; }
//client* xevent::data() { return cb->data; }
//int xevent::no() { return cb->n; }
//char* xevent::id() { return cb->it; }

event* xevent::get() const { return (event*) new ::tcl_event(this); }

int xevent::x() const {
if (last) 
   switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.x;
	case MotionNotify:
		return LAST->xmotion.x;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.x;
	default: return -1;
	}
else return 0;
}

int xevent::y() const {
if (last) 
    switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.y;
	case MotionNotify:
		return LAST->xmotion.y;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.y;
	default: return -1;
	}
else return 0;
}

int xevent::x_root() const {
if (last)
switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.x_root;
	case MotionNotify:
		return LAST->xmotion.x_root;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.x_root;
	default: return -1;
	}
else return 0;
}

int xevent::y_root() const {
if (last)
switch (LAST->type) {
	case ButtonPress:
	case ButtonRelease:
		return LAST->xbutton.y_root;
	case MotionNotify:
		return LAST->xmotion.y_root;
	case KeyPress:
	case KeyRelease:
		return LAST->xkey.y_root;
	default: return -1;
	}
else return 0;
}

int xevent::button(int i) const {
const int n = buttno();
if ( ( type() == ButtonPress ) && ( i==0?1:n==i ) )  return n;
else return 0;
//return (( type() == ButtonPress ) && ( i==0?1:buttno()==i ) ) ? buttno : 0; 
}

int xevent::buttonup(int i) const {
const int n = buttno();
if ( ( type() == ButtonRelease ) && ( i==0?1:n==i ) )  return n;
else return 0;
//return (( type() == ButtonRelease ) && ( i==0?1:buttno()==i ) ) ? buttno : 0;
}

int xevent::keycode() const {
if (last && keyevent())
	return LAST->xkey.keycode; else return -1; 
}

int xevent::buttonevent(int i) const {
return buttevent() && ( i==0?1:buttno()==i ); 
}

int xevent::keyevent() const {
const int tp = type(); return (tp==KeyPress) || (tp==KeyRelease); 
}

int xevent::motion() const { return (type() == MotionNotify); }

// private:

int xevent::buttevent() const {
return ((type()==ButtonPress) || (type()==ButtonRelease));
}

int xevent::buttno() const {
if (last && buttevent()) return LAST->xbutton.button; else return -1;
}

#endif
