// handler.c

#include <hush/handler.h>


#include <hush/kit.h>
#include <hush/event.h>
#include <hush/types.h>

#include <hush/assert.h>

#include <dv/port.h>

#include <string.h>
#include <stdio.h>



static event a_dummy_event;
event* dummy_event = & a_dummy_event;


handler::handler() : _event(0), _etc(0), tk(0), _mode(0) {
	tk = thekit();
	}

int handler::mode() const { return _mode; }
int handler::mode( int i ) { int tmp = _mode; _mode = i; return tmp; }
int handler::mode( const char* s ) {
	int tmp = _mode;
	if (!strcmp(s,"explicit")) _mode = handler::explic;
	else if (!strcmp(s,"implicit")) _mode = handler::implicit;
	else cerr << "ERROR: " << s << " no such handler::mode" << endl;
	return _mode;
	}

handler::~handler() {}

char* handler::category() const  { return "handler"; }
char* handler::name() const  { return "anonymous"; }
char* handler::info() const  { return "handler"; }

kit* handler::thekit() const  { return tk?tk: (::thekit()); }

event* handler::dispatch(event* e) {
	 
	 _event = e;
	 //require (tk);

	 if (e) tk = e->thekit();

	 if (tk && tk->tracelevel() > 5)
	     cerr << "invoking handler " << tk->info() << endl;

	 int result = this->operator()();

	 if (tk && tk->tracelevel() > 5)
	 	cerr << "exiting handler " << endl;

	 if (!result) return 0;
	 else return _event;
}

int handler::operator()() {

#ifndef HCLONLY

	 require( tk );

	 if (!_event) return 0;    /// check for event

	 event& e = * _event;

	 if (tk->tracelevel() > 5) {
	 	cerr << "EVENT: " << e.eventclass() << e.info() << endl;
		}

	 if (tk->tracelevel() > 3) {
	   cerr << "event" << e.type() << " " << e.name() << endl;
	   }

	 if ( e.type() == 0 || e.type() == ButtonPress ) press(e);
	 else if ( e.type() == ButtonRelease ) release(e);
	 else if ( e.type() == KeyPress ) keypress(e);
	 else if ( e.type() == KeyRelease ) keyrelease(e);
	 else if ( e.type() == MotionNotify ) motion(e);
	 else if ( e.type() == EnterNotify ) enter(e);
	 else if ( e.type() == LeaveNotify ) leave(e);
	 else other(e);

#endif /* HCLONLY */

	 //delete &e;  AE!
	 
	 return 0;

	 }

void handler::other(event&) { cerr << "**other event**" << endl; }

int handler::verify() const { return 1; }
void handler::obsolete(const char* s) const {
	cerr << "OBSOLETE: " << s << endl;
		}





STATIC
struct h_upd_list {  
h_upd_list(handler* hd=0, h_upd_list* tl=0) : el(hd), next(tl) {}
~h_upd_list() { if(next) delete next; }
int contains(handler* p, handler* e);
void insert(handler* e);
h_upd_list* remove(handler* e);
void process(event* e);
handler* el;
h_upd_list* next;
};


void handler::dependent(handler* e) {
	if (e->contains(this)) {
	  cerr << "ERROR: circular dependency by linking ";
	  cerr << e->name() << " to " << this->name() << endl;
	}
	else if (!_etc) _etc = (void*) new h_upd_list(e,0);
	else ((h_upd_list*)_etc)->insert(e);
	}

void handler::remove(handler* e) {
if (!e && _etc) { delete _etc; _etc = 0; } 
else if (_etc) _etc = (void*) ((h_upd_list*)_etc)->remove(e);
}

void handler::process(event* e) {
	if (_etc) ((h_upd_list*)_etc)->process(e);
	}

int handler::contains(handler* e) {
if (_etc) return ((h_upd_list*)_etc)->contains(this,e);
else return 0;
}

int h_upd_list::contains(handler* p, handler* e) {
require(e && el);
if (e == p) return 1;
else if (el == e) return 1;
else if (el->contains(e) ) return 1;
else if (!next) return 0;
else return next->contains(p,e);
}
 
void h_upd_list::insert(handler* e) { 
        if (!el) el = e;
        else if (!next) next = new h_upd_list(e);
        else next->insert(e);
        }

h_upd_list* h_upd_list::remove(handler* e) { 
	if (next) next = next->remove(e);
        if (el == e) return next;
        else return this;
        }

void h_upd_list::process(event* e) {
        if (el) el->dispatch(e);
	if (next) next->process(e);
        }



int handler::_no = 0;


char* handler::flatten(int argc, char* argv[]) const {
static char buf[BUFSIZ];
char temp[BUFSIZ];
buf[0]='\0';
if (argc > 0)  { strcpy(buf,*argv++); argc--; }
while (--argc >= 0) {
	temp[0] = '\0';
	sprintf(temp,"%s %s",buf,*argv++);
	strcpy(buf,temp);
        }
return buf;
}

static int check(const char* s) {
int b = 0; const char* p = s;
while (!b && *p) if (*p++ == ' ') b = 1;
return b;
}


char* handler::quote(int argc, char* argv[]) const {
static char buf[BUFSIZ];
char temp[BUFSIZ];
buf[0]='\0';
if (argc > 0)  { strcpy(buf,*argv++); argc--; }
while (--argc >= 0) {
	temp[0] = '\0';
	if (check(*argv))
		sprintf(temp,"%s {%s}",buf,*argv++);
	else
		sprintf(temp,"%s %s",buf,*argv++);
	strcpy(buf,temp);
        }
return buf;
}


char* handler::gensym(const char* s) const {
static char buf[BUFSIZ];
if (!s)
  sprintf(buf,"handler-%d",++handler::_no);
else
  sprintf(buf,"%s-%d",s,++handler::_no);
return buf;
}
