// string.c

#include <hush/string.h>

#include <hush/event.h>
#include <hush/list.h>


#include <stdio.h>

#include <iostream.h>
#include "string.h"

const int MaxWArgs = 36;
const int MaxLArgs = 256;

static iter<string> dummy_string_iter(0);
static list<string> dummy_string_list;
static list< iter<string> > dummy_string_iter_list;


class dvstringrep {
friend class dvstring;
friend class dvstring_iter;
private:
dvstringrep(const char* s);
dvstringrep(int argc_,  char* argv_[]);
char* flatten(int i);
int unflatten();
int convert();
int assoc();
void mkassoc();

void _register(string* s);

void split(const char* s);

~dvstringrep();
char* rep;
int count;
int _argc;
char** _argv;
char* _key;
char* _value;
int _mode;
list<string>* _list;
list<void>* _garb;
list<void>* _igarb;
};

#define REP ((dvstringrep*)_body)

class dvstring_iter : public iter<dvstring> {
public:
dvstring_iter(dvstringrep* r);
dvstring* operator()();
private: 
int n;
dvstringrep* _r;
};


void dvstringrep::_register(string* s) {
	if (!_garb) _garb = new dvlist<void>;
	_garb->insert( s );
	}

dvstringrep::dvstringrep(const char* s) : _mode(string::plain) {
	rep = new char[strlen(s)+1]; strcpy(rep,s);
	count = 1; _key = 0; _value = 0; _list = 0;
	_garb = 0; _igarb = 0;
	}

dvstringrep::dvstringrep( int argc_, char** argv_) : _mode(string::argcv) {
count = 1; _key = 0; _value = 0;
_argc = argc_;
_argv = argv_;
rep = 0;
_list = 0;
_garb = 0;
_igarb = 0;
}


dvstringrep::~dvstringrep() { 
	if (count<=0) {
	  if (rep) delete rep; 
	  if (_mode == string::pair) {
		if (_key) delete _key;
		if (_value) delete _value;
		}
	  if (_list) {
		dvstring* sp = (dvstring*) _list->first();
		while (sp) {
		      delete sp;
		      sp = (dvstring*) _list->next();
		      }
		delete _list;
	  }     
	  if (_garb) {
		dvstring* sp = (dvstring*) _garb->first();
		while (sp) {
		      delete sp;
		      sp = (dvstring*) _garb->next();
		      }
		delete _garb;
	  }     
	  if (_igarb) {
		dvstring_iter* sp = (dvstring_iter*) _igarb->first();
		while (sp) {
		      delete sp;
		      sp = (dvstring_iter*) _igarb->next();
		      }
		delete _igarb;
	  }     
	}
}


dvstring::dvstring() { _body = (void*) new dvstringrep(""); }
dvstring::~dvstring() { if (--REP->count <= 0) delete REP; }
dvstring::dvstring(const char* s) { _body =  new dvstringrep(s); }
dvstring::dvstring(const dvstring& a) { 
	   _body = a._body; REP->count++; 
	   } 

dvstring::dvstring(int argc_, char** argv_) {
	_body = new dvstringrep(argc_,argv_);
	}

dvstring::dvstring(const event* e) {
       _body = new dvstringrep(e->argc(), e->argv());
       }

dvstring& dvstring::operator=( const dvstring& a ) { 
	   REP->count++;
	   if (--REP->count <= 0 )
		delete REP;
	   _body = a._body;
	   return *this;
	   }

int dvstring::length() const { return strlen(REP->rep); } 

dvstring::operator char*() const { return REP->rep; }

dvstring dvstring::operator+( const dvstring& a ) const { 
	char* tmp = new char[length() + a.length()];
	strcpy(tmp,REP->rep); strcat(tmp, (char*) a);
	dvstring result( tmp ); delete[] tmp;
	return result;
	}

dvstring dvstring::operator+( const char* a ) const { 
	char* tmp = new char[length() + strlen(a) + 1];
	strcpy(tmp,REP->rep); strcat(tmp, a);
	dvstring result( tmp ); delete[] tmp;
	return result;
	}

int dvstring::operator==( const dvstring& a ) const { 
	return !strcmp(REP->rep,(char*) a );
	}

int dvstring::operator==( const char* a ) const { 
	return !strcmp(REP->rep,a);
	}

dvlist<dvstring>* dvstring::split(const char* sp) {
	REP->split((char*)sp);
	return (dvlist<dvstring>*) REP->_list;
	}

dvstring_iter::dvstring_iter(dvstringrep* r)
			: _r(r), iter<dvstring>(this) {
	n = 0;
	if (!_r->_igarb) _r->_igarb = new dvlist< void >;
	_r->_igarb->insert(this);
	}

dvstring* dvstring_iter::operator()() {
	dvstring* sp = 0; 
	if ( _r->_argc > n ) {
		sp = new dvstring( _r->_argv[n] );
		_r->_register(sp);
		n++;
		}
	return sp;
	}




dvstring::operator iter<dvstring>&() {
	if (REP->_list) return (iter<dvstring>&) * REP->_list; 
	else if (((dvstring*)this)->argc() > 0)
	   return * (iter<dvstring>*) new dvstring_iter(REP); 
	else return (iter<dvstring>&) dummy__iter;
}


int dvstring::argc() {
	   if (! REP->_mode) unflatten();
	   return REP -> _argc; 
	   }
char** dvstring::argv( int i ) {
	   if (! REP->_mode) unflatten();
	   if (i > REP->_argc)
	      return ( REP -> _argv ) + i;
	   else return 0;
	   }

dvstring dvstring::arg( int i ) {
	    if (! REP->_mode) unflatten();
	    if (i < REP->_argc) 
	    return REP -> _argv[i]; 
	    else return "";
	    }

dvstring& dvstring::flatten(int i) {
	    REP->flatten(i);
	    return *this;
	    }

int dvstring::unflatten() { return REP->unflatten(); }

int dvstring::assoc() { return REP->assoc(); }
dvstring dvstring::key() { return assoc()?REP->_key:""; }
dvstring dvstring::value() { return assoc()?REP->_value:""; }

static int check(const char* s) {
int b = 0; const char* p = s;
while (!b && *p) if (*p++ == ' ') b = 1;
return b;
}



char* dvstringrep::flatten( int b ) {
int argc_ = _argc;
char** argv_ = _argv;
char *temp; 
int n = BUFSIZ;
int m = 0;
temp = new char[n];
temp[0]='\0';
if (argc_ > 0)  { m = strlen(*argv_); strcpy(temp,*argv_++); argc_--; }
while (--argc_ >= 0) {
        m += strlen(*argv_);
	if ( m > n ) {
		char* tn = new char[n]; 
		strcpy(tn,temp);
		n *= 2;
		delete temp;
		temp = new char[n];
		strcpy(temp,tn);
		delete tn;
		}
	if (b && check(*argv_)) 
	    sprintf(temp,"%s {%s}",temp,*argv_);
	else
	    strcat(temp,*argv_);
	argv_++;
        }
rep = new char[strlen(temp)+1]; strcpy(rep,temp);
delete temp;
return rep;
}






int dvstringrep::unflatten() {
char* in = rep;
static char argv_[MaxWArgs][MaxLArgs];
_argv = (char**) argv_;
int brackets = 0;
int quoteon = 0; int cont = 1;
char inbuf[BUFSIZ];
strcpy(inbuf,in);
char* b = inbuf;
while (*b == ' ') b++;
char* p = b;
int i = 0;
while ( *p != '\0' ) {
i++;
char prev = '\0';
quoteon = 0; cont = 1; /// quotes[quoteon]
while ( *p != '\0' && cont ) {
	if ( *p == ' ' && !quoteon && !brackets ) cont = 0;
	else if ( *p == '"' && prev != '\\') quoteon = quoteon?0:1;
	else if ( *p == '{' && prev != '\\') brackets++;
	else if ( *p == '}' && prev != '\\') brackets--;
	if (cont) { prev = *p;  p++; }
	}

if ( *p == '\0' ) {
	strcpy(argv_[i-1],b);
	return i;
} else {
	*p = '\0';
	strcpy(argv_[i-1],b);
	p++;
	b = p;
	}
}
_argc = i;
return i;
}


int dvstringrep::assoc() {
if (_mode == string::pair) return 1;
else {
  int res = 0;
  for ( char* p = rep; *p ; p++ ) {
	if (*p == '=') res = 1;
	}
  if (res) mkassoc();
  return res;
  }
}



void dvstringrep::mkassoc() {
int m = strlen(rep);
char* _buf = new char[m+1];
strcpy(_buf,rep);
char* p = _buf;
while ( *p != '=' ) p++;
*p = '\0';
if ( *(p+1) == '"' ) p++;
_key = new char[strlen(_buf)+1];
_key[0]='\0';
strcpy(_key,_buf);
_value = new char[strlen(p+1)+1];
strcpy(_value,p+1);
int n = strlen( _value ) - 1;
if (_value[n]=='"') _value[n]='\0';
_mode = string::pair;
}

static int charin(char c, const char* in) {
char* p = (char*) in;
while ( *p ) {
	if ( c == *p ) return 1;
	p++;
	}
return 0;
}


void dvstringrep::split(const char* s) {
_list = new dvlist<string>;
char* p = rep;

while (*p) {
	char buf[BUFSIZ];
	char* q = buf;

	while ( *p && !charin(*p,s) ) {
		*q = *p; p++; q++;
		}
	if (*p) while( charin(*p,s) ) p++;
	*q = '\0';
	dvstring* tmp = new dvstring( buf );
	_list->insert(tmp);
	}
}
