// widget.c -- abstract widget class

#include <hush/widget.h>
#include <hush/action.h>


extern void register_as(const char*,const widget*);

#include <hush/assert.h>

#include <string.h>

#include <tk.h>

typedef class handler ancestor;

int widget::widgetno = 0;

char* widget::info() const { return this->type(); }
char* widget::type() const { return "widget"; }

void widget::title(const char* s) { 
	sprintf(buf,"wm title %s {%s}",path(),s);
	tk->eval( buf );
	}

#ifndef HCLONLY
extern void* thewindow(kit*,const char*);
extern void* themainwindow();
#endif

void widget::alias(const widget* top) {  // default 0
	if (!top || top == this) {
		sprintf(buf, "rename %s %s-win", path(), path());
		tk->eval( buf );
		}
        tk->bind( path(), this );
        if (top && top != this ) register_as( path(), (widget*) top );
	tk->result( path() );
}

void widget::image(const char* s) { 
        if (s[0] != '@')
                sprintf(buf,"%s configure -image %s",path(),s);
        else {
                sprintf(buf,"image create photo -file %s",s+1);
                char* r = tk->evaluate(buf);
                sprintf(buf,"%s configure -image %s",path(),r);
                }
        tk->eval(buf);
        }



int widget::width()  const { return Tk_ReqWidth((Tk_Window)this->tkwin()); }
int widget::height() const { return Tk_ReqHeight((Tk_Window)this->tkwin()); }

widget::widget() { tk = thekit(); _self = 0; }

widget::widget(const char* s) {
	path(s);
	tk = thekit();
	require( tk );
	_self = 0;
	buf = new char[BUFSIZ];
	}

widget::widget(const widget* w, const char* s) {
	path(w,s);
	tk = thekit();
	_self = 0;
	buf = new char[BUFSIZ];
	}

widget::~widget() {
	if (tk->tracelevel()>6) cerr << "D: (W) " << path() << endl;
	if (buf) delete[] buf;
	if (_path) delete[] _path;
}

char* widget::path() const { return  _path; }

int widget::eval(const char* s) {
     sprintf(buf,"%s %s",thepath(),s);	
     return tk->eval(buf);
     }

char* widget::result() const { return tk->result(); }

char* widget::evaluate(const char* cmd) { eval(cmd); return result(); }


void widget::destroy() { 
	sprintf(buf,"destroy %s",path());
	tk->eval( buf );
	}

void widget::xscroll(const widget* s) {
	sprintf(buf,"%s configure -xscrollcommand {%s set}",thepath(),((widget*)s)->self()->path());
	// s->xview( self() );
	tk->eval( buf );
	}

void widget::yscroll(const widget* s) {
	sprintf(buf,"%s configure -yscrollcommand {%s set}",thepath(),((widget*)s)->self()->path());
	// s->yview( self() );
	tk->eval( buf );
	}


void widget::configure(const char* s) { 
	sprintf(buf,"%s configure %s",thepath(),s);
	tk->eval( buf );
	}

void widget::geometry(int x, int y) {
	sprintf(buf,"%s configure -width %d -height %d",thepath(),x,y);
	tk->eval( buf );
	}


widget* widget::pack(const char* options) {
        if ((!strcmp(options,".quit")) || (!strcmp(options,".quit")))
		return this->pack(".quit","-side bottom -fill x");
        else if (!strcmp(options,"quit"))
		return this->pack(".quit","-side bottom -fill x");
	else if (!strcmp(options,"."))
		return this;
	else {
		sprintf(buf,"pack %s %s", path(), options);
		tk->eval(buf);
	}
	return this;
	}

widget* widget::pack(widget* w, const char* options) {
	if (options[0]=='{') 
  	  sprintf(buf,"pack append %s %s %s", path(), w->path(),
	  options);
	else 
	  sprintf(buf,"pack configure %s %s", w->path(), options);
	tk->eval(buf);
	return this;
	}

widget* widget::pack(const char* w, const char* options ) {
	if (!strcmp(w,"quit")) 
	    return pack(".quit",options);
	else if (!strcmp(w,".quit")) {
	    require(tk);
  	    tk->eval("button .quit -text quit -command quit"); 
	    }
	if (options[0]=='{') 
  	  sprintf(buf,"pack append %s %s %s", path(), w, options);
	else 
	 sprintf(buf,"pack %s %s", w, options);
	tk->eval(buf);
	return this;
	}

void widget::place(const char* options) {
	sprintf(buf,"place %s %s",path(), options);
	tk->eval(buf);
	}


widget::bind(const char *s, const char* cmd, const char* args) {
	action* ac = tk->action(cmd);
	return direct(s,ac,args);
	}

widget::bind(const char *s, void* proc, void* data, const char* args ) {
	if (!data) data = this;
	action* ac = tk->action(proc,data);
	return direct(s,ac,args);
	}

widget::bind(const char *s, class handler* h, const char* args) {
	action* ac = tk->action(h);
	return direct(s,ac,args);
	}

widget::bind(const char *s, action* ac, const char* args) {
        return direct(s,ac,args);
	}

// Remember, for GNU's sake ...


widget::bind(void* proc, void* data, const char* args) {
	if (!data) data = this;
	action* ac = tk->action(proc,data);
	install(ac, args);
	return 0;
	}

widget::bind(class handler* h, const char* args) {
	action* ac = tk->action(h);
	install(ac,args);
	return 0;
	}

widget::bind(action* ac, const char* args) {
	install(ac,args);
	return 0;
	}

#ifndef NOHANDLER
void widget::handler(const char* cmd, const char* args) {
	action* ac = tk->action(cmd);
	install(ac,args);
	}

void widget::handler(void* proc, void* data, const char* args) {
	if (!data) data = this;
	action* ac = tk->action(proc,data);
	install(ac, args);
	}

void widget::handler(class handler* h, const char* args) {
	require( h );
	action* ac = tk->action(h);
	install(ac,args);
	}

void widget::handler(action* ac, const char* args) {
	install(ac,args);
	}
#endif


void widget::source(const char* type, const char* args, const char* options) {
	sprintf(buf,"drag&drop source %s -button 3 -tokencmd {package_%s %s} -sendcmd send_%s %s",
	  path(),type,args,type,options);
	  tk->eval( buf );
	  }

void widget::target(const char* type, action* ac, const char* args, const char* options) {
	sprintf(buf,"drag&drop target %s handler {%s {%s %s}} %s",
	  path(),type,ac->name(),args,options);
	  tk->eval( buf );
	  }

void widget::focus(const char* options) {
	sprintf(buf,"focus %s %s", options, path()); 
	tk->eval( buf );
	}

void widget::grab(const char* options) {
	sprintf(buf,"grab %s %s", options, path()); 
	tk->eval( buf );
	}

void widget::wait() {
	sprintf(buf,"tkwait window %s",path());
	tk->eval( buf );
	}

#ifndef HCLONLY
//AE void widget::map() { Tk_MapWindow( (Tk_Window) tkwin() ); }
//void widget::unmap() { Tk_UnmapWindow( (Tk_Window) tkwin() ); }

void* widget::tkwin() const { return thewindow(tk,path()); }
#endif

void widget::prefix(const widget* p) {
    path(p,path());
}


widget* widget::self() const { 
	return (widget*) (exists()?_self->self():this);
	}

int widget::exists() const { return (_self && _self != this)?1:0; }

widget* widget::redirect(widget* w) {
	widget* t = _self; _self = w; return t;
	}

char* widget::thepath() const { return self()->path(); }

void widget::install(action* , const char*) { }

widget::direct(const char *s, action* a, const char* args) {
	sprintf(buf,"bind %s %s { %s %s }", thepath(),s,a->name(),args);
	return tk->eval( buf );
	}

void widget::path( const char* s) {
	_path = new char[strlen(s)+2];
	if (s[0] != '.') sprintf(_path,".%s",s);
	else sprintf(_path,"%s",s);
	}

void widget::path( const widget* w, const char* s) { 
	char* tmp = new char[strlen(w->path()) + strlen(s) + 4];
	if (s[0] != '.' && s[0] != '-' && s[0] != '_')
			sprintf(tmp,"%s.%s",w->path(),s);
	else sprintf(tmp,"%s%s",w->path(),s);
	if( tmp[0]=='.' && tmp[1]=='.' ) tmp += 1;
	_path = tmp;
	}


void widget::init(const char* options) {
	if (options) {
		sprintf(buf,"%s %s %s",type(),path(),options);
		tk->eval( buf );
		}
	}

//// Garbage collection -- advanced

void widget::_register(widget* w) {
	cerr << "NYET widget:;_register " << w->path() << endl;
	}

int widget::operator()() {
	return ancestor::operator()();
	}

    // default interpreter
    /*
       int operator()() {
       cout << "widget: " << argv[0] << " (" << type() << ")" << endl;
       if ( !strcmp( type() ,*argv) )  create(--argc,++argv);
       else if (!strcmp("self",argv[1]) ) tk->result(self()->path());
       else self()->eval( flatten(--argc,++argv) );
       return OK;
       }
       virtual void create(int ,char*[]) {}
       */


