//<h3>event.h</h3>
//The event class is crucial for hush and the
//support for discrete event simulation.
//It's implementation is a bit complex.
//<hr> event(4), <hush/hush.h>, <hush/handler.h>, <hush/kit.h> <hr>

//<ignore>
#ifndef HUSH_EVENT_H
#define HUSH_EVENT_H


//// class declarations

class widget;
class kit;
class event;
class histogram;
//</ignore>

/// the event class -- a truly overloaded entity

#include <hush/handler.h>


//<html>
//<hr>
//</html>


//// ? Will I be fat or lean ?

class event  : public handler {  /// B/W
int _del;
public:


//// Event classes

enum { user = 0, system = 1, tcl, file, sim, tt, vr, nop };

public: // should be protected, and may be will be

event(kit*,int,char**);             // tcl event

event(int qp=0,int sp=0,int k=0);   // simevent

event(const char* s);               // modevent
event(const event&);

//// to allow for object references

event(event*);                       // xevent
virtual ~event();     //// if (_body && _del) delete ev;

/////////// dispatching ///////////

virtual int operator()();

public:    //// common methods ///////////////////

virtual int eventclass() const ; /// defaults to 0 = user

virtual int type() const;   /// defaults to 0 
virtual char* name() const ; 

virtual void trace() const; 

//// To obtain 'the' event, allows for 'type secure' event handling

virtual event* get() const;

//// obsolete

virtual int no() const;
virtual char* id() const;

//// etelosbo

//////// tcl / file event methods -- for tcl callbacks

virtual int argc() const;
virtual char** argv(int i = 0) const;
virtual char* arg(int i = 0) const;
virtual int fd() const;
virtual int mask() const;

//////// xevent methods -- for windows /////////////

virtual int x() const ;
virtual int y() const ;
virtual int x_root() const ;
virtual int y_root() const ;
virtual int button(int i = 0) const ;
virtual int buttonup(int i = 0) const ;
virtual int keycode() const ;
virtual int buttonevent(int i = 0) const ;
virtual int keyevent() const ;
virtual int motion() const ;
virtual void motion( event& );
virtual void* rawevent() ;

/////////////// simevent -- for simulation //////////

virtual int report(histogram* hi,double interv = 0);
virtual void stamp();
virtual double timespent();

virtual char* reporttitle();
virtual double time() const;
virtual void time(double t);
virtual int schedulingpriority();
virtual void schedulingpriority(int p);
virtual int queuingpriority();
virtual void queuingpriority(int p);
virtual double interval();
virtual void interval(double i);
virtual int state();
virtual void state(int s);
virtual int killflag();
virtual event* leftpointer();
virtual void leftpointer(event*);
virtual event* middlepointer();
virtual void middlepointer(event*);
virtual event* rightpointer();
virtual void rightpointer(event*);
virtual event* parentpointer();
virtual void parentpointer(event*);
virtual double waitingreporttime();
virtual void waitingreporttime(double);
virtual double eventreporttime();
virtual int active();
virtual int pending();
virtual int conditional();
virtual int closed();
virtual int passive();
virtual int queued();


//// obsolete
//////// modevent methods -- for modeling /////////////

/*
virtual void dependent(event* e); // to define dependent events 
virtual void remove(event* e); // to remove dependent events
virtual int contains(event* e); // to check for cycles
virtual void update(); // to activate dependent events/handlers
*/

//// etelosbo

////////////// the body --- also very weakly protected, still

private:
int exists() const; /// to check for circularity;
event* _body;
};

//<html>
//<hr>
//</html>


#endif
