//<h3>handler.h</h3>
//Defines the class handler, which is
//an ancestor of events as well as widgets
//and items, as expressed in the 
//See the
//<a href=http://www.cs.vu.nl/~eliens/hush/structure.html#Widgets>class hierarchy</a>
//<hr> [], [index],  handler(4), <hush/hush.h> <hr>

//<ignore>
#ifndef HANDLER_H
#define HANDLER_H

#ifdef __GNUG__
#define NOHANDLER
#endif

#include <hush/ok.h>

class event;
class kit;
class file_event;
class tcl_event;

extern kit* thekit();

//</ignore>


class handler { // : public client {
static int _no;                    // to keep count of handlers
public:

enum  { implicit = 0, explic }; /// modes -- GNU does not allow explicit!



handler();
virtual ~handler();

virtual char* category() const;    /// handler 
virtual char* name() const;    /// handler 
virtual char* info() const;    /// anonymous

//// Implicit

virtual event* dispatch(event* e = 0);
virtual int operator()();

//// Explicit

virtual int handle_tcl_event( tcl_event* ) { return 1; }
virtual int handle_file_event( file_event* ) { return 1; }

//// Default methods (for canvas):

virtual void press( event&  ) { }
virtual void release( event&  ) { }
virtual void keypress( event&  ) { }
virtual void keyrelease( event&  ) { }
virtual void motion( event&  ) { }
virtual void enter( event&  ) { }
virtual void leave( event&  ) { }
virtual void other( event& );

//// Access to the (local) kit:

virtual kit* thekit() const;

//// Modalities

virtual int mode() const;
virtual int mode(int);
virtual int mode(const char*);

//// Dependent handlers:

virtual void dependent(handler* e); // to define dependent handlers 
virtual void remove(handler* e); // to remove dependent handlers
virtual int contains(handler* e); // to check for cycles
virtual void process(event* e = 0); // to activate dependent handlers

protected:
event* _event;
kit* tk;
private:
int _mode;
void* _etc;

public: /// utilities

virtual char* flatten(int argc, char* argv[]) const;
virtual char* quote(int argc, char* argv[]) const;
virtual char* gensym(const char* s=0) const;

virtual void obsolete(const char* msg = "HANDLER") const; 
virtual int verify() const;
};

//<html>
//<hr>
//</html>

#endif
