//<h3>kit.h</h3>
//The kit class defines the interface between C++ and Tcl.
//<hr>
//<a href=hush.html>hush</a>,
// kit(4), <hush/handler.h>
// examples/hush/kit.c, api/hush/kit.c <hr>

//<ignore>
#ifndef KIT_H
#define KIT_H

#include <hush/handler.h>

class handler;
class widget;
class kit;


//</ignore>

//// Export

extern void thekit(kit*);
extern kit* thekit();


//// Class interface

class kit : public handler {
protected:

public:

enum { tcl = 0, dlp, python, unknown };

kit(const char* = "tcl");
kit(kit*);
virtual ~kit();

//// Evaluation:

virtual int eval(const char* cmd);
virtual char* evaluate(const char* cmd);

virtual char* result() const;
virtual void result(const char* s);

virtual int source(const char* f);

virtual int error(); /// N E W
virtual void error(const char*); // N E W

virtual void after(int n, const char* cmd="");
virtual void after(int n, void*, void* = 0);
virtual void after(int n, handler* h);

virtual char* send(const char* it, const char* cmd);
virtual char* selection(const char* options="") const;
virtual void update(const char* options="");


virtual widget* root() const; 
virtual widget* pack(widget* w, const char* options = "-side top -fill x");
virtual widget* pack(const char* w, const char* options = "-side top -fill x");


//// Binding Actions: high level:

enum {readable=1, writable=2, exception=4}; // jrvosse

virtual void bind(const char* name, handler* h);
virtual void bind(int fd, handler* h, int mask=readable); // jrvosse

virtual void unbind(int fd);
virtual void unbind(char* name, char* newname = "");


virtual void trace(int i = 1);
virtual void notrace();
virtual int  tracelevel() const;

virtual void quit();

//// Multiple interpreters

virtual int safe() const;
virtual kit* slave(char* options = "");

//// Actions: low  level:

virtual class action* action(const char*);
virtual class action* action(const char* name, handler* h);
virtual class action* action(const char* name, void*, void* data = 0 );
virtual class action* action(int fd, handler* h, int mask=readable);

//// Anonymous actions

virtual class action* action(handler* h);
virtual class action* action(void*, void* data = 0 );

//// Commands: lowest  level:  ???

virtual void command(int fd, void*, void*, int mask=readable); //jrvosse
virtual void command(const char* name, void*, void*); /// N E W

//// Miscelaneous:

virtual void* interp() const; /// the interpreter, whatever it is
virtual void* tkwin() const;  /// returns the root window

//// System-level:

public: /// undocumented

virtual int mode() const;
virtual int mode(int fl);
virtual int mode(const char* = 0);

virtual int map(widget*,int i = 1) { return i; } /// to map or unmap widgets
virtual int callback(void*,void*,int,char**); /// ???

// You can play numerous tricks with events:

virtual event* get() const;
virtual void put(event* p, int fl = 0); /// to update events

//// Experimental -- role switching

kit* become(char* s, kit* k = 0);
void restore(kit* k = 0);

//// Obsolete:

public: // obsolete ?

virtual void bind(const char* name, void* , void* = 0);

/// anonymous

private:
kit* tk;
};

#endif

//<hr>See kit(4), examples/hush/kit.c and api/hush/h-kit.c.
