// list.h -- (api/hush/list.c)

#ifndef _DVLIST_H
#define _DVLIST_H

#include <hush/iter.h>
#include <iostream.h>


//<ignore>
struct dvlist_cell {  
dvlist_cell(void* hd=0, dvlist_cell* tl=0);
~dvlist_cell();

int empty() const;
int length() const;

int count(void* x) const;
int contains(void* e) const;

void insert(void* e);
dvlist_cell* remove(void* e);

iter<void>* search() const;


void* el;
dvlist_cell* next;


//// Garbage collection

int _cnt;  
int _level;

void _register(iter<void>*);
void _register(dvlist_cell*);

dvlist_cell* _igarb;
dvlist_cell* _lgarb;
};
//</ignore>

//// Class  list:

template< class E > 
class dvlist {
public:
friend ostream& operator<<(ostream& os, dvlist<E>& dvl);
typedef ostream& dvlist_format_function_type(ostream& os, E* e);
static ostream& dvlist_default_format(ostream& os, E* e) {
os << e << endl;
return os;
}

dvlist() { c = new dvlist_cell(); init(); }
dvlist( const dvlist<E>& dvl) {
	dvl.c->_cnt++;
	c = dvl.c;
	}

~dvlist() {
        if (_lgarb != 0) delete _lgarb;
	if (_level == 0 && c ) delete c;
	}


virtual dvlist<E>* clone() {
	dvlist<E>* tmp = new dvlist<E>;
	iter<E> it = *this; /// obtain iter by assignment
	E* p = 0;
	while ( p = it() ) tmp->insert(p);
	tmp->format( dvlist_format_function );
	return tmp; // no registration and, hence, no deletion
	}

dvlist<E>& operator=(const dvlist<E>& dvl) {
	dvlist_cell* tmp = c;
	dvl.c->_cnt++;
	c = dvl.c;
	if ( c->_cnt == 0 ) delete c;
	return (*this);
	}

virtual int operator==(const dvlist<E>& dvl) { return 0; }
virtual int operator<=(const dvlist<E>& dvl) { return 0; }
virtual int operator<(const dvlist<E>& dvl) { return 0; }

int empty() const { return c->empty(); }
int length() const { return c->length(); }


int contains(const E* el) const { return c->contains((void*) el); }
int count(const E* el) const { return c->count((void*) el); }

virtual void insert(const E* el) { c->insert((void*) el); }
virtual void remove(const E* el) { c = c->remove((void*) el); }

dvlist<E>& operator<<(const E* e) { this->insert(e); return *this; }

//// Access

E* head() const { return (E*) c->el; }
dvlist<E>* tail() const { 
	dvlist<E>* tmp = new dvlist(c->next); 
	tmp->_level++;
	((dvlist<E>*)this)->_register(tmp);
	return tmp;
	}

//// Iterator

operator iter<E>&() const { return * (iter<E>*) c->search(); } 

	
//// Cursor

E* current() { return _current?(E*)_current->el:0; }  
E* first() { _current = c; return current(); }
E* next() { 
	_current = _current? (_current = _current->next):0;
	return current();
	}

/// Printing

void print(ostream& os) const {
	iter<E> it = *this;
	E* p = 0; 
	while ( p = it() )  {
		dvlist_format_function(os,p);
		}
}

void format(dvlist_format_function_type* fun) {
	dvlist_format_function =
		  (dvlist_format_function_type*) fun;  /// have fun
	}

//// Processing
	
int process( int fun(E* e) ) {  /// side-effects!
        int tmp = 0;
	E* p = first();
	while (p) {
		tmp += fun(p); /// do the thing
		p = next();
		}
	return tmp;
	}

//// Mapping

dvlist<E>* map( E* fun(E* e) ) {  /// side-effects with element
        dvlist<E>* tmp = new dvlist<E>;
	tmp->format( dvlist_format_function );
	E* p = first();
	while (p) {
		tmp->insert( fun(p) ); /// do the thing
		p = next();
		}
	return tmp;
	}

dvlist<E>* map( E* fun(E*, dvlist<E>*) ) {  /// with list
        dvlist<E>* tmp = new dvlist<E>;
	tmp->format( dvlist_format_function );
	E* p = first();
	while (p) {
		tmp->insert( fun(p,this) ); /// do the thing
		p = next();
		}
	_register(tmp);
	return tmp;
	}

/// Applying (go one level higher)

#ifdef might_be_recursive
dvlist< dvlist<E> >* apply( dvlist<E>* fun(E*) ) {  
        dvlist< dvlist<E> >* tmp = new dvlist< dvlist<E> >;
	E* p = first();
	while (p) {
		dvlist<E>* x = fun(p); /// do it, and 
		x->format( dvlist_format_function );
		_register(x); // register for removal
		tmp->insert( x ); 
		p = next();
		}
	_register( (dvlist<E>*) tmp;
	return tmp;
	}  // deletion is done automatically
#endif // might_be_recursive

#ifdef might_be_recursive
dvlist< dvlist<E> >* apply( dvlist<E>* fun(E*, dvlist<E>*) ) {  
        dvlist< dvlist<E> >* tmp = new dvlist< dvlist<E> >;
	E* p = first();
	while (p) {
		dvlist<E>* x = fun(p,this);
		x->format( dvlist_format_function );
		_register(x);
		tmp->insert( x ); 
		p = next();
		}
	_register( (dvlist<E>*) tmp); // trick to register
	return tmp;
	} // deletion is done automatically
#endif // might_be_recursive



void _register(dvlist<E>* p) { // to clean up afterwards
	if (!_lgarb) _lgarb = new dvlist_cell;
	if (!_lgarb->contains(p)) _lgarb->insert(p); // to avoid dups
	}

private:
int _level;
void init() {
	    _current = c;  _lgarb = 0;  _level = 0; 
	    dvlist_format_function = dvlist_default_format;
	    }

dvlist(dvlist_cell* x) { c = new dvlist_cell(x->el,x->next); init(); }
dvlist_cell* _current;
dvlist_cell* c;
dvlist_cell* _lgarb;
dvlist_format_function_type*  dvlist_format_function;
};


// OPERATORS

template<class T >
inline
dvlist<T>& operator<<(dvlist<T>& in, T* e) {
in.insert(e); return in;
}

template<class T >
inline
ostream& operator<<(ostream& os, dvlist<T>& in) {
in.print(os); return os;
}

#ifndef list
#define list dvlist
#endif

#ifndef dvlist_format_function_type
#define list_format_function_type dvlist_format_function_type
#endif



#endif
