//<html>
//<h3>widget.h</h3>
//Defines the abstract class <i>widget</i>.
//Abstract in a general sense, not in a technical C++ sense.
//<hr>
//<a href=hush.html>hush</a>,
// widget(4),
//<a href=kit.h.html>kit.h</a>,
//<a href=handler.h.html>handler.h</a>,
//<a href=action.h.html>action.h</a>,
//<hr>
//</html>

//<ignore>
#ifndef WIDGET_H
#define WIDGET_H

#include <stdio.h>
#include <hush/handler.h>
#include <hush/kit.h>

class action;

//</ignore>

class widget : public handler {
  public:
    widget(const char* s);
    widget(const widget* w, const char* s);
    
    virtual ~widget();
    
    char* info() const;
    virtual char* type() const ;
    
    char* path() const ;
    
    int eval(const char* s);
    char* result() const;
    char* evaluate(const char* cmd);
    
    void destroy();

    virtual void image(const char*);
    
    virtual void configure(const char* s);
    virtual void geometry(int x, int y);
    
    virtual widget* pack(const char* options = "-fill x" );
    virtual widget* pack(widget* w, const char* options = "-side top -fill x -fill y" ) ;
    
    virtual widget* pack(const char* w, const char* options );
    
    virtual bind(const char *s, const char* cmd,  const char* args = "");
    virtual bind(const char *s, handler* h, const char* args = "" );
    virtual bind(const char *s, action* a,  const char* args = "" );
    virtual bind(const char *s, void* proc, void* data = 0, 
				 	const char* args = "" ) ;
    
    
    // For GNU's sake ...
    
    //virtual bind(char* cmd, char* args = "");
    virtual bind(::handler* h, const char* args = "" );
    virtual bind(action* ac,   const char* args = "" );
    virtual bind(void* proc, void* data = 0, const char* args = "" ) ;
    
    // Will not be needed after v2.6, but no doubt maintained ...
    
#ifndef NOHANDLER
    void handler(class handler* h, const char* args = "" );
    void handler(action* ac, const char* args = "" );
    void handler(const char* cmd, const char* args = "");
    void handler(void* proc, void* data = 0, const char* args = "" );
#endif
    
    void source(const char* type, const char* args, const char* options="");
    void target(const char* type, action* ac, const char* args, const char* options="");
    
    void xscroll(const widget* s);
    void yscroll(const widget* s);
    
    void focus(const char* options = "");
    void grab(const char* options = "");
    void wait();
    
    virtual void* tkwin() const;
    
    int width() const ;
    int height() const;
    
    widget* self() const;
    
    void alias( const widget* w = 0 );


    // Grabage collection -- advanced

    void _register(widget*);
    
  protected:
    widget();
    static int widgetno;

    int exists() const;
    
    
    widget* redirect(widget* w);
    char* thepath() const;
    
    virtual void install(action* , const char*);
    
    virtual void init(const char* options);
    
    void path( const char* s);
    void path( const widget* w, const char* s);
    
    char* buf;
    class kit* tk;
    
    
  public:

    int operator()();
    
  private:
    widget* _self;
    char* _path;
    
  public:
    virtual direct(const char *s, action* a, const char* args = "" );
    
    // obsolete
    
    void prefix(const widget* p);
    virtual void place(const char* options); // obsolete
    
    //void map();
    //void unmap();
    
    void title(const char* s);
};


//<ignore>
#endif
//</ignore>
//<hr> api/hush/widget.c
