#include <hush/hush.h>
#include <widgets/canvas.h>
#include <dv/tk/citem.h>

#include <string.h>
#include <hush/assert.h>
#include<stdlib.h>

static item* new_c_item(int n, canvas* x) {
	return new item( new c_item(n,x) );
	}

char* canvas::type() const { return "canvas"; }

canvas::canvas(const char *p, const char* options) : widget(p) {
	if (options) init(options);
	} 

canvas::canvas(const widget* w, const char *p, const char* options) : widget(w,p) { 
	if (options) init(options);
	} 

void canvas::region(int x1, int y1, int x2, int y2 ) {
	sprintf(buf,"%s configure -scrollregion { %d %d %d %d }",
		thepath(),x1,y1,x2,y2);
	tk->eval( buf );
	}


int canvas::canvasx(int screenx) { // returns x relative to canvas
                sprintf(buf, "%s canvasx %d", thepath(), screenx);
                if (0==tk->eval(buf)) return atoi(tk->result());
                else return -1;
            }

int canvas::canvasy(int screeny) { // returns y relative to canvas
                sprintf(buf, "%s canvasy %d", thepath(), screeny);
                if (0==tk->eval(buf)) return atoi(tk->result());
                else return -1;
            }

void canvas::install(action* ac, const char* args) {
	bind("<Any-ButtonPress>",ac, args);
	bind("<Any-ButtonRelease>",ac, args);
	bind("<Any-Motion>",ac, args );
	bind("<Enter>",ac, args );
	bind("<Leave>",ac, args );
	}

void canvas::tag(int id, const char* tag) {
	sprintf(buf,"%s addtag %s withtag %d",thepath(),tag,id);
	tk->eval(buf);
	}

char* canvas::tags(int id) {
	sprintf(buf,"%s gettags %d",thepath(),id);
	return tk->evaluate(buf);
	}

void canvas::del(int id) {
	sprintf(buf,"%s delete %d",thepath(),id);
	tk->eval(buf);
	}

void canvas::del(const char* id) {
	sprintf(buf,"%s delete %s",thepath(),id);
	tk->eval(buf);
	}

void canvas::move(int id, int x, int y) {
	sprintf(buf,"%s move %d %d %d",thepath(),id,x,y);
	tk->eval(buf);
	}

void canvas::move(const char* id, int x, int y) {
	sprintf(buf,"%s move %s %d %d",thepath(),id,x,y);
	require(tk);
	tk->eval(buf);
	}

item& canvas::bitmap(int x1, int y1, const char* bitmap, const char* options ) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	if (bitmap[0] == '@' )
	sprintf(buf,"%s create bitmap %d %d -bitmap \"%s\" %s",thepath(),x1,y1,bitmap,options);
	else
	sprintf(buf,"%s create bitmap %d %d -bitmap \"%s\" %s",thepath(),x1,y1,bitmap,options);

	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::image(int x1, int y1, const char* bitmap, const char* options ) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	if (bitmap[0] == '@' ) {
	sprintf(buf,"image create photo -file %s",bitmap+1);
	char* r = tk->evaluate( buf );
	sprintf(buf,"%s create image %d %d -image \"%s\" %s",thepath(),x1,y1,r,options);
	} else
	sprintf(buf,"%s create image %d %d -image \"%s\" %s",thepath(),x1,y1,bitmap,options);

	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::photo(int x1, int y1, int x2, int y2, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	x2 = canvasx( x2 ); y2 = canvasy( y2 );
	sprintf(buf,"%s create photo %d %d %d %d %s",thepath(),x1,y1,x2,y2,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

void canvas::readppm(int id, const char* file) {
	sprintf(buf,"readppm %s-%d %s",thepath(),id,file);
	tk->eval( buf );
	}

item& canvas::line(int x1, int y1, int x2, int y2, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	x2 = canvasx( x2 ); y2 = canvasy( y2 );
	sprintf(buf,"%s create line %d %d %d %d %s",thepath(),x1,y1,x2,y2,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::line(const char* linespec, const char* options) {
	sprintf(buf,"%s create line %s %s",thepath(),linespec,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::arc(int x1, int y1, int x2, int y2, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	x2 = canvasx( x2 ); y2 = canvasy( y2 );
	sprintf(buf,"%s create arc %d %d %d %d %s",thepath(),x1,y1,x2,y2,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::oval(int x1, int y1, int x2, int y2, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	x2 = canvasx( x2 ); y2 = canvasy( y2 );
	sprintf(buf,"%s create oval %d %d %d %d %s",thepath(),x1,y1,x2,y2,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::circle(int x1, int y1, int r, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	sprintf(buf,"%s create oval %d %d %d %d %s",thepath(),x1-r,y1-r,x1+r,y1+r,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::polygon(const char* linespec, const char* options) {
	sprintf(buf,"%s create polygon %s %s",thepath(),linespec,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::rectangle(int x1, int y1, int x2, int y2, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	x2 = canvasx( x2 ); y2 = canvasy( y2 );
	sprintf(buf,"%s create rectangle %d %d %d %d %s",thepath(),x1,y1,x2,y2,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::text(int x1, int y1, const char* txt, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	sprintf(buf,"%s create text %d %d -text \"%s\" %s",thepath(),x1,y1,txt,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::window(int x1, int y1, const char* win, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	sprintf(buf,"%s create window %d %d -window \"%s\" %s",thepath(),x1,y1,win,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::window(int x1, int y1, widget* win, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	sprintf(buf,"%s create window %d %d -window \"%s\" %s",thepath(),x1,y1,win->path(),options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::sphere(int x1, int y1, int x2, int y2, const char* color, const char* tag, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	x2 = canvasx( x2 ); y2 = canvasy( y2 );
	sprintf(buf,"vsphere %s %s %d %d %d %d %s %s",thepath(),tag,x1,y1,x2,y2,color,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::sphere(int x1, int y1, int r, const char* color, const char* tag, const char* options) {
	x1 = canvasx( x1 ); y1 = canvasy( y1 );
	sprintf(buf,"vsphere %s %s %d %d %d %d %s %s",thepath(),tag,x1-r,y1-r,x1+r,y1+r,color,options);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}

item& canvas::current() {
	sprintf(buf,"%s find current",thepath());
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}
	
item& canvas::overlapping(int x, int y, const char* options) {
	char ids[BUFSIZ];  
	x = canvasx( x ); y = canvasy( y );
	strcpy(ids, items(x,y));
	int n;
	if (!strcmp(options,"first")) {
		sscanf(ids,"%d",&n);
		}
	else {
		//AE p = ids.lindex( ids.llength() ); sscanf(p,"%d",&n);
		n = 0;
		}
	return *(new_c_item(n,this));
	}

item& canvas::overlapping(int x, int y) {
	x = canvasx( x ); y = canvasy( y );
	sprintf(buf,"%s find overlapping %d %d %d %d",thepath(),x,y,x,y);
	int n = 0; sscanf(tk->evaluate(buf),"%d",&n);
	return *(new_c_item(n,this));
	}


char* canvas::items(int x, int y) {
	x = canvasx( x ); y = canvasy( y );
	sprintf(buf,"%s find overlapping %d %d %d %d",thepath(),x,y,x,y);
	return tk->evaluate( buf );
	}

void canvas::itemconfigure(int it, const char* options) {
	sprintf(buf,"%s itemconfigure %d %s",thepath(),it,options);
	tk->eval(buf);
	}

void canvas::itemconfigure(const char* tag, const char* options) {
	sprintf(buf,"%s itemconfigure %s %s",thepath(),tag,options);
	tk->eval(buf);
	}

canvas::itembind(int i, const char* s, action* a, const char* args) {
        return itemdirect(i,s,a,args);
	}

canvas::itembind(const char *i, const char* s, action* a, const char* args) {
        return itemdirect(i,s,a,args);
	}

void canvas::postscript(const char* file, const char* options) {
	sprintf(buf,"%s postscript -file %s %s",thepath(), file, options);
	tk->eval( buf );
	}


canvas::itemdirect(int i, const char *s, action* ac, const char* args) {
	sprintf(buf,"%s bind %d %s { %s %s }", thepath(),i,s,ac->name(),args);
	return tk->eval( buf );
	}

canvas::itemdirect(const char* i, const char *s, action* ac, const char* args) {
	sprintf(buf,"%s bind %s %s { %s %s }", thepath(),i,s,ac->name(),args);
	return tk->eval( buf );
	}
