#include <hush/hush.h>
#include <widgets/frame.h>
#include <widgets/scrollbar.h>
#include <widgets/hypertext.h>

#include <string.h>

hypertext::hypertext() { }

hypertext::hypertext(const char* p, const char* options) : widget( p ) 
{
    if (options) init(options);
}

hypertext::hypertext(const widget* w, const char* p, const char* options) : widget(w,p) 
{ 
    if (options) init(options);
}

hypertext::~hypertext() {}

char* hypertext::type() const { return "hypertext"; }

void hypertext::file(const char* f) {
    sprintf(buf,"%s configure -file %s", thepath(), f);
    tk->eval( buf );
}

void hypertext::geometry(int x, int y) {
    sprintf(buf,"%s configure -geometry %dx%d",thepath(),x,y);
    tk->eval( buf );
}



void hypertext::configure(const char* s) { 
    sprintf(buf,"%s configure %s",thepath(),s);
    tk->eval( buf );
}

void hypertext::init(const char* options) {
    
    //widget* top = new frame( this, "-ht", " -class HyperText " );
    widget* top = new frame( path(), " -class HyperText " );
    
    sprintf(buf,"htext %s.htext %s", top->path(), options);
    tk->eval( buf );
    
    widget* c = new widget(top,".htext");
    
    scrollbar* scrollx = new scrollbar(top,".scrollx");
    scrollx->orient("horizontal");
    scrollx->pack("-side bottom -fill x");
    
    scrollbar* scrolly = new scrollbar(top,".scrolly");
    scrolly->orient("vertical");
    scrolly->pack("-side right -fill y");
    
    sprintf(buf,"%s.htext configure "
	    "-yscrollcommand { %s.scrolly set } "
	    "-xscrollcommand { %s.scrollx set } "
	    "-yscrollunits 10m -xscrollunits 1i ", 
	    path(), path(), path() );
    
    tk->eval( buf );
    
    c->pack();
    
    redirect(c);
    
    c->xscroll(scrollx); scrollx->xview(c);
    c->yscroll(scrolly); scrolly->yview(c);
    
    alias( this );
    tk->result( path() );
}


int hypertext::operator()() { 
    int argc = _event->argc(); char** argv = _event->argv();
    if (!strcmp("self",argv[1]) ) tk->result(self()->path());
    else if ( !strcmp( "hypertext" ,*argv) )
	create(--argc,++argv); 
    else self()->eval( quote(--argc,++argv) );
    return 0;
}

void hypertext::create(int arc, char* args[]) { 
    const char* name = args[0];
    new hypertext(name, quote(--arc,++args));
}

