#include <widgets/menu.h>
#include <hush/action.h>

#include <string.h>

static char* empty = "";

char* menu::type() const { return "menu"; }

menu::menu(const char* p, const char* options) : widget(p) {
	if (options) init(options); 
	_name = empty;
	_args = empty;
}

menu::menu(const widget* w, const char* p, const char* options) : widget(w,p) {
	if (options) init(options);
	_name = empty;
	_args = empty;
}

menu* menu::oldadd(const char* s, const char* options) {
	sprintf(buf,"%s add %s %s", path(),s,options);
	tk->eval( buf );
	return this;
	}

menu* menu::entry(const char* s, action* ac, const char* args,const char* options) {
	return add(s,ac,args,options);
	}

menu* menu::entry(const char* s, const char* args, const char* options) {
	return add(s,args,options);
	}

menu* menu::add(const char* s, action* ac, const char* args, const char* options) {
	sprintf(buf,"%s add command -label \"%s\" -command \"catch {%s {%s} %s}\" -state active %s",
		path(),s,ac->name(),s,args,options);
	tk->eval( buf );
	return this;
	}

menu* menu::add(const char* s, const char* args, const char* options) {
	sprintf(buf,"%s add command -label \"%s\" -command \"catch {%s {%s} %s %s}\" -state active %s",
		(char*)path(),s,_name,s,args,(char*) _args,options);
	tk->eval( buf );
	return this;
	}

menu* menu::checkbutton(const char* s, const char* args, const char* options) {
        sprintf(buf,"%s add checkbutton -label \"%s\" -command \"catch {%s {%s} %s %s}\" -state active %s", (char*)path(),s,_name,s,args,(char*) _args,options);
        tk->eval( buf );
        return this;
}    


menu* menu::cascade(const char* s, const char* m, const char* options) {
	sprintf(buf,"%s add cascade -label \"%s\" -menu %s %s",
		path(),s,m,options);
	tk->eval( buf );
	return this;
	}

menu* menu::cascade(const char* s, menu* m, const char* options) {
	sprintf(buf,"%s add cascade -label \"%s\" -menu %s -state active %s",
		path(),s,m->path(),options);
	tk->eval( buf );
	return this;
	}

char* menu::entryconfigure(int i, const char* options) {
	sprintf(buf,"%s entryconfigure %d %s",path(),i,options);
	return tk->evaluate( buf );
	}

int menu::index(const char *s) const {
	sprintf(buf,"%s index %s",path(),s);
	int n = -1; sscanf(tk->evaluate( buf ),"%d",&n);
	return n;
	}

int menu::active() const { return index("active"); }


void menu::del(int i) {
	sprintf(buf,"%s delete @%d",path(),i);
	tk->eval( buf );
	}

void menu::del(const char* s) {
	sprintf(buf,"%s delete %s",path(),s);
	tk->eval( buf );
	}

char* menu::invoke(int i) {
	sprintf(buf,"%s invoke @%d",path(),i);
	return tk->evaluate( buf ); 
	}

char* menu::invoke(const char *s ) {
	sprintf(buf,"%s invoke %s",path(),s);
	return tk->evaluate( buf );
	}

void menu::post(int x, int y) {
	sprintf(buf,"%s post %d %d",path(),x,y);
	tk->eval( buf );
	}

void menu::unpost() {
	sprintf(buf,"%s unpost",path());
	tk->eval( buf );
	}

void menu::install(action* ac, const char* args) {
	_name = new char[strlen(ac->name())+1];
	strcpy(_name,ac->name());
	_args = new char[ strlen(ac->name())+1];
	strcpy(_args,args);
	}
